@@ -1,11 +1,11 @@
+import assert from 'assert';
 import { X2jOptions, XMLParser } from 'fast-xml-parser';
 import * as fs from 'fs';
+import { cpus } from 'os';
 import * as path from 'path';
 import * as vscode from 'vscode';
 import { CancellationToken } from 'vscode-languageclient';
-import { parallelize, staggerProgress } from './helpers';
-import { cpus } from 'os';
-import { assert } from 'console';
+import { getMatchingPrefixes, parallelize, staggerProgress, toPosix } from './helpers';
 
 /**
  * TypeScript types to represent data from GNATcoverage XML reports
@@ -299,25 +299,122 @@ export async function addCoverageData(run: vscode.TestRun, covDir: string) {
             progress.report({
                 message: `${done} / ${totalFiles} source files`,
             });
+
+            let posixForeignPrefix: string | undefined;
+            let posixLocalPrefix: string | undefined;
+
+            const procs = process.env.PROCESSORS ? Number(process.env.PROCESSORS) : 0;
             const fileCovs = (
                 await parallelize(
                     array,
-                    Math.min(cpus().length, 8),
+                    Math.min(procs == 0 ? cpus().length : procs, 8),
                     async (file) => {
                         if (token.isCancellationRequested) {
                             throw new vscode.CancellationError();
                         }
 
                         assert(file['@_name']);
+                        const foreignPath = file['@_name'];
+                        /**
+                         * The foreign machine may have a different path
+                         * format, so we normalize to POSIX which is valid on
+                         * both Windows and POSIX OS-es.
+                         */
+                        const posixForeignPath = toPosix(foreignPath);
 
-                        let srcUri: vscode.Uri;
-                        if (path.isAbsolute(file['@_name']!)) {
-                            srcUri = vscode.Uri.file(file['@_name']!);
-                        } else {
+                        let srcUri: vscode.Uri | undefined = undefined;
+
+                        /**
+                         * The goal here is to find the file in the workspace
+                         * corresponding to the name attribute in the GNATcov
+                         * report.
+                         *
+                         * The name could be a basename (older GNATcov
+                         * versions) or an absolute path (newer GNATcov
+                         * versions).
+                         *
+                         * In the case of a basename, the only course of action
+                         * is to do a file lookup in the workspace.
+                         *
+                         * In the case of an absolute path, the path
+                         * corresponds to the machine where the report was
+                         * created which might be a foreign machine or the
+                         * local host. We can't know in which situation we are
+                         * so it's best to assume that it's a foreign machine.
+                         *
+                         * Then the logic consists of searching for the first
+                         * file by basename, which gives a local absolute path.
+                         * Then by comparing the local absolute path and the
+                         * foreign absolute path, we can find a foreign prefix
+                         * path that should be mapped to the local prefix path
+                         * to compute local absolute paths. Subsequent files
+                         * can use the computed prefixes directly without a
+                         * workspace lookup.
+                         */
+
+                        if (path.posix.isAbsolute(posixForeignPath)) {
+                            let localFullPath;
+                            if (posixLocalPrefix && posixForeignPrefix) {
+                                /**
+                                 * The prefixes have already been determined, so
+                                 * use them directly.
+                                 */
+
+                                if (posixForeignPath.startsWith(posixForeignPrefix)) {
+                                    // Extract the relative path based on the foreign prefix
+                                    const posixForeignRelPath = path.relative(
+                                        posixForeignPrefix,
+                                        posixForeignPath,
+                                    );
+
+                                    // Resolve the relative path with the local prefix
+                                    localFullPath = path.join(
+                                        posixLocalPrefix,
+                                        posixForeignRelPath,
+                                    );
+                                }
+                            }
+
+                            // Fallback to using the input path as is
+                            localFullPath = localFullPath ?? foreignPath;
+
+                            if (fs.existsSync(localFullPath)) {
+                                srcUri = vscode.Uri.file(localFullPath);
+                            }
+                        }
+
+                        if (srcUri === undefined) {
+                            /**
+                             * GNATcov generates instrumented versions of the
+                             * sources with the same basenames. We want to
+                             * avoid associating coverage data with the
+                             * instrumented sources, so we exclude any paths
+                             * containing the special directory
+                             * `gnatcov-instr`. Ideally it would have been nice
+                             * to exclude precisely `<obj-dir>/gnatcov-instr`
+                             * but that would need to be repeated for each obj
+                             * dir of each project in the closure. As we don't
+                             * have access to that information, we ignore all
+                             * paths containing a `gnatcov-instr` component.
+                             *
+                             * Note that a previous version excluded the entire
+                             * object dir which did not work well on projects
+                             * that use '.' as the object dir. In that case
+                             * excluding the object dir would exclude the
+                             * entire workspace and prevent finding any files.
+                             */
+                            const exclude = `**/gnatcov-instr/**/*`;
+
+                            /**
+                             * If the prefixes haven't been found yet, or
+                             * the last prefixes used were not successful,
+                             * try a workspace lookup of the basename.
+                             */
                             const found = await vscode.workspace.findFiles(
-                                `**/${file['@_name']!}`,
-                                null,
+                                `**/${path.posix.basename(posixForeignPath)}`,
+                                exclude,
                                 1,
+                                token,
                             );
                             if (found.length == 0) {
                                 return undefined;
@@ -326,6 +423,27 @@ export async function addCoverageData(run: vscode.TestRun, covDir: string) {
                             srcUri = found[0];
                         }
 
+                        if (
+                            posixForeignPrefix === undefined &&
+                            posixLocalPrefix === undefined &&
+                            path.posix.isAbsolute(posixForeignPath)
+                        ) {
+                            /**
+                             * If the prefixes haven't been calculated, and the
+                             * foreign path is absolute, let's try to compute
+                             * the prefixes based on the workspace URI that was
+                             * found.
+                             */
+
+                            const localAbsPath = srcUri.fsPath;
+                            const posixLocalAbsPath = toPosix(localAbsPath);
+
+                            [posixForeignPrefix, posixLocalPrefix] = getMatchingPrefixes(
+                                posixForeignPath,
+                                posixLocalAbsPath,
+                            );
+                        }
+
                         const total = file.metric.find(
                             (m) => m['@_kind'] == 'total_lines_of_relevance',
                         )!['@_count'];
@@ -334,14 +452,15 @@ export async function addCoverageData(run: vscode.TestRun, covDir: string) {
                         ];
 
                         const fileReportBasename = data.coverage_report.sources!['xi:include'].find(
-                            (inc) => inc['@_href'] == `${path.basename(file['@_name']!)}.xml`,
+                            (inc) =>
+                                inc['@_href'] == `${path.posix.basename(posixForeignPath)}.xml`,
                         )!['@_href'];
                         const fileReportPath = path.join(covDir, fileReportBasename);
 
                         if (covered > total) {
                             throw Error(
                                 `Got ${covered} covered lines for a` +
-                                    ` total of ${total} in ${file['@_name']!}`,
+                                    ` total of ${total} in ${file['@_name']}`,
                             );
                         }
 
